#include "stm32f4xx.h"
#include "SysTick.h"
#include "DUART.h"
#include "SDCard.h"
#include "diskio.h"
#include "ff.h"

/* 注意把PE3接到VCC防止LIS302DL干扰. */
FATFS fs;            // Work area (file system object) for logical drive
FIL fsrc, fdst;      // file objects
BYTE buffer[512]; // file copy buffer
FRESULT res;         // FatFs function common result code
UINT br, bw;         // File R/W count

#if _USE_LFN
char Fs_LongFileName[_MAX_LFN * (0x81 ? 2 : 1) + 1];
#endif

uint32_t total = 0;
uint32_t free = 0;

uint8_t f_getfreespace(uint32_t *total,uint32_t *free)
{
	FATFS *fs1;
	uint8_t res;
    uint32_t fre_clust=0, fre_sect=0, tot_sect=0;
    //得到磁盘信息及空闲簇数量
    res =(uint32_t)f_getfree((const TCHAR*)"0", (DWORD*)&fre_clust, &fs1);
    if(res==0)
	{											   
	    tot_sect=(fs1->n_fatent-2)*fs1->csize;	//得到总扇区数
	    fre_sect=fre_clust*fs1->csize;			//得到空闲扇区数	   
#if _MAX_SS!=512				  				//扇区大小不是512字节,则转换为512字节
		tot_sect*=fs1->ssize/512;
		fre_sect*=fs1->ssize/512;
#endif	  
		*total=tot_sect>>1;	//单位为KB
		*free=fre_sect>>1;	//单位为KB 
 	}
	return res;
}

void OutPutFile(void)
{ 
	unsigned int a;
	FRESULT res;
	FILINFO finfo;
	DIR dirs;
	char path[16]={""};  
#if _USE_LFN
    finfo.lfname = Fs_LongFileName;
    finfo.lfsize = sizeof(Fs_LongFileName);
#endif

	printf("Init Disk:%d\n",disk_initialize(0));
	Delay(5);
	printf("Disk Mount:%d\n",f_mount(&fs,"0:",1));
	f_getfreespace(&total,&free);
	printf("Total: %d MB,Free %d MB\n",total/1024,free/1024);
	Delay(5);
	if (f_opendir(&dirs, path) == FR_OK) 
	{
    	while (f_readdir(&dirs, &finfo) == FR_OK)  
    	{
      		if (finfo.fattrib & AM_ARC) 
      		{
        		if(!finfo.fname[0]) break;        
        		printf("SFN:%s\r\n",finfo.fname);
#if _USE_LFN
        		printf("LFN:%s\r\n",finfo.lfname);
#endif
        		res = f_open(&fsrc, finfo.fname, FA_OPEN_EXISTING | FA_READ);
        		br=1;
				a=0;
						printf("File Preview:");
						for(a=0; a<512; a++) buffer[a]=0; 
		    	    res = f_read(&fsrc, buffer, sizeof(buffer), &br);
					printf("%s\r\n",buffer);	
		    	    if (res || br == 0) break;   // error or eof
				f_close(&fsrc);                 
			}
		} 
	}
  
	while(1);
}

int main(void)
{    
  RCC->AHB1ENR = 0x001000FF;
  SysTick_Init();
  DUSART_Init();
  
  OutPutFile();
}

