#include "TimerCAP.h"

void TIM13_CAP_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
    NVIC_InitTypeDef NVIC_InitStructure;
    TIM_ICInitTypeDef  TIM13_ICInitStructure;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM13, ENABLE);
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

    GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_TIM13);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    TIM_TimeBaseStructure.TIM_Prescaler = 83;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseStructure.TIM_Period = 0XFFFFFFFF;
    TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;

    TIM_TimeBaseInit(TIM13, &TIM_TimeBaseStructure);

    TIM13_ICInitStructure.TIM_Channel = TIM_Channel_1;
    TIM13_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
    TIM13_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
    TIM13_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
    TIM13_ICInitStructure.TIM_ICFilter = 0x00;
    TIM_ICInit(TIM13, &TIM13_ICInitStructure);

    TIM_ITConfig(TIM13, TIM_IT_Update | TIM_IT_CC1, ENABLE);

    TIM_Cmd(TIM13, ENABLE );

    NVIC_InitStructure.NVIC_IRQChannel = TIM8_UP_TIM13_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

}

uint8_t  TIM13CH1_CAPTURE_STA = 0;
uint32_t	TIM13CH1_CAPTURE_VAL;

void TIM8_UP_TIM13_IRQHandler(void)
{

    if((TIM13CH1_CAPTURE_STA & 0X80) == 0) //还未成功捕获
    {
        if(TIM_GetITStatus(TIM13, TIM_IT_Update) != RESET)//溢出
        {
            if(TIM13CH1_CAPTURE_STA & 0X40) //已经捕获到高电平了
            {
                if((TIM13CH1_CAPTURE_STA & 0X3F) == 0X3F) //高电平太长了
                {
                    TIM13CH1_CAPTURE_STA |= 0X80;		//标记成功捕获了一次
                    TIM13CH1_CAPTURE_VAL = 0XFFFFFFFF;
                }
                else TIM13CH1_CAPTURE_STA++;
            }
        }
        if(TIM_GetITStatus(TIM13, TIM_IT_CC1) != RESET)//捕获1发生捕获事件
        {
            if(TIM13CH1_CAPTURE_STA & 0X40)		//捕获到一个下降沿
            {
                TIM13CH1_CAPTURE_STA |= 0X80;		//标记成功捕获到一次高电平脉宽
                TIM13CH1_CAPTURE_VAL = TIM_GetCapture1(TIM13); //获取当前的捕获值.
                TIM_OC1PolarityConfig(TIM13, TIM_ICPolarity_Rising); //CC1P=0 设置为上升沿捕获
            }
            else  								//还未开始,第一次捕获上升沿
            {
                TIM13CH1_CAPTURE_STA = 0;			//清空
                TIM13CH1_CAPTURE_VAL = 0;
                TIM13CH1_CAPTURE_STA |= 0X40;		//标记捕获到了上升沿
                TIM_Cmd(TIM13, DISABLE ); 	//关闭定时器5
                TIM_SetCounter(TIM13, 0);
                TIM_OC1PolarityConfig(TIM13, TIM_ICPolarity_Falling);		//CC1P=1 设置为下降沿捕获
                TIM_Cmd(TIM13, ENABLE ); 	//使能定时器5
            }
        }
    }
    TIM_ClearITPendingBit(TIM13, TIM_IT_CC1 | TIM_IT_Update); //清除中断标志位
}
